module jsim2 
#(parameter DATA_WIDTH=16, parameter ADDR_WIDTH=5)
(
	input clk, reset_n,
	output [(ADDR_WIDTH-1):0] addr,
	output we,
	output [2:0] ra, rb, rd,
	output  [(DATA_WIDTH-1):0] q, qa, qb, qd,
	output reg [4:0] pc,
	output reg [(DATA_WIDTH-1):0] inst,
	output [2:0] opcode,
	//output reg [2:0] state, 
	output ma, rwe
);

single_port_ram dev1(
	.data(qb),
	.addr(addr),
	.we(we),
	.clk(clk),
	.q(q)
);

assign rwe = state==WB;

regfile dev2(
	.clk(clk),
	.we(rwe),
	.ra(ra),
	.qa(qa),
	.rb(rb),
	.qb(qb),
	.rd(rd),
	.qd(qd)
);

assign ma = memory_access; // make variable name shorter

//reg [4:0] pc;,

//wire [(DATA_WIDTH-1):0] q;
//wire [2:0] opcode;
wire [15:0] qi = (state==ID? q: inst);
assign opcode = qi[15:13];
assign rd = qi[12:10]; // CODE DELETED
assign ra = qi[9:7];
assign rb = (op_sw||op_bne? qi[12:10]: qi[2:0]);

reg [2:0] state;
localparam 
	IF = 3'h0,
	ID = 3'h1,
	EX = 3'h2,
	MEM = 3'h3,
	WB = 3'h4,
	HALT = 3'h7;
	
`include "opcodes.txt"

wire op_sw = opcode==SW;
wire op_lw = opcode==LW;
wire op_bne = opcode==BNE;
wire op_jalr = opcode==JALR;

reg [(ADDR_WIDTH-1):0] jump;

wire memory_access;
assign we = (state==MEM) && op_sw;
assign memory_access = (op_sw || op_lw);

wire do_branch = 0; // CODE DELETED


wire [4:0] nextpc = pc + 1'b1;
wire [4:0] newpc = (op_jalr||do_branch? jump: nextpc);

assign addr = (state==MEM? d1: newpc );


wire [15:0] d1;

alu dev3(
  .a(qa),
  .b(qb),
  .pc({11'h000,nextpc}),
  .im10(qi[9:0]),
  .d(d1),
  .opcode(opcode)
);

// writeback register multiplexer
assign qd = (op_lw? q: d1);


always @(posedge clk, negedge reset_n)
begin
  if (!reset_n)
  begin
   pc <= 5'h1F;
   state <= IF;
   end
  else 
	case (state)
	IF:
	begin
		state <= ID;
		pc <= addr;
    end
    ID:
    begin
	 // save results of IF phase
	 inst <= q;
	 state <= EX;
	end
	EX:
	 begin
	  jump <= 0; // CODE DELETED
	  if (inst == 16'he071) state <= HALT;
      else if (memory_access)
      begin
		state <= MEM;
	  end
      else 
      begin
		state <= WB;
	  end
	end
    MEM:
    begin
	   state <= WB;
	end
	WB:
	begin
	   state <= IF;
	end
	default:
		state <= HALT;
  endcase
end

endmodule
